//
// simple_net.h
// rev1. Mon, 13 Sep 2010 00:01:39 -0300 - first release
//
// DiegoSpinola @ Hackeneering.com
//

/*
  ____________ ______ _______ __________________ ________
 |            |      |       |                  |        |
 | Address (2)| Cmd  | Psize | PayLoad( Psize ) | ChkSum |
 |____________|______|_______|__________________|________|

*/

//Packet Handler
#define ADD_POS 0
#define CMD_POS 2
#define PSI_POS 3
#define PAY_POS 4

#define BROADCAST1 0xFF
#define BROADCAST2 0xFF

#define MYADD1 ((MODULE_ID >> 8 )&0xFF) //Own Address MSB
#define MYADD2 ((MODULE_ID)      &0xFF) //Own Address LSB

//CMDs
//Default commands (all must implement)
#define CMD_WHO_ARE_YOU 0x01       //returns the 32bit module Id ( see module ID format )
#define CMD_ACK  0x02          	   //returns possible command return values (wow! amazing!)
#define CMD_ADDRESS_CHANGE 0x03    //changes the address of this host and records this in nvm

void packet_handler(unsigned int8 * packet);
int1 AddressCheck( unsigned int8 * packet , int8 AddressHigh, int8 AddressLow);
void send_msg(unsigned int8 destH , unsigned int8 destL ,unsigned int8 cmd, unsigned int8 * payload , unsigned int8 size);

static int1 ReceivedAck=0;

void clear_Ack_count()
{
   ReceivedAck=0;
}

int1 get_Ack_count()
{
   return ReceivedAck;
}

/*
  ____________ ______ _______ __________________ 
 |            |      |       |                  |
 | Address (2)| Cmd  | Psize | PayLoad( Psize ) |
 |____________|______|_______|__________________|

*/

#define PSIZE packet[PSI_POS]

void app_packet_handler(unsigned int8 * packet,  int1 isBroadcast);

void packet_handler(unsigned int8 * packet)
{
   int1 isBroadcast=0;
   int1 isAddMatch=0;
   int8 reply[4];
  //check if it ours, otherwise ignore
  isBroadcast=AddressCheck(packet,0xFF,0xFF);
  isAddMatch =AddressCheck(packet,MYADD1,MYADD2);
  
  
  
  if( !(isBroadcast || isAddMatch) )return;
  
  switch( packet[CMD_POS])
   {
   // This is where all default commands parsing is done
     case CMD_WHO_ARE_YOU:
   reply[0]= (MODULE_ID >> 24) & 0xFF;
   reply[1]= (MODULE_ID >> 16) & 0xFF;
   reply[2]= (MODULE_ID >> 8 ) & 0xFF;
   reply[3]= (MODULE_ID >> 0 ) & 0xFF;
   if(!isBroadcast)send_msg(0xFF,0xFF,CMD_ACK,reply,4); //Ack to the message
      break;
	
     case CMD_ACK: //TODO: STORE THE RESPONSE DATA SOMEWHERE (implement when needed)
      ReceivedAck=1;
     break;

     case CMD_ADDRESS_CHANGE:
	if(isAddMatch){
         write_eeprom (HIGH_ADD_EEPROM_ADD, packet[PAY_POS]);
         write_eeprom ( LOW_ADD_EEPROM_ADD, packet[PAY_POS+1]);
		 send_msg(0xFF,0xFF,CMD_ACK,packet,0); //Ack to the message
	}
     break;
	 
	// Application specific below this line
	default:
		app_packet_handler( packet,  isBroadcast);
	break;
	//Made this into a pure virtual function app_packet_handler (declared in this module but implemented in the user code somewhere else)
/*
    case CMD_SET_PWM_TARGET:
     destination_pwm= packet[PAY_POS];
     if(!isBroadcast)send_msg(0xFF,0xFF,CMD_ACK,packet,0); //Ack to the message
     break;
    case CMD_SET_PWM_SPEED:
     next_change_period= packet[PAY_POS]*0x100+packet[PAY_POS+1];
     if(!isBroadcast)send_msg(0xFF,0xFF,CMD_ACK,packet,0); //Ack to the message
     break;
*/	 
	//End of Application specific parsing
   }
}

int1 AddressCheck( unsigned int8 * packet , int8 AddressHigh, int8 AddressLow)
{
   if(packet[ADD_POS]  != AddressHigh)return 0;
   if(packet[ADD_POS+1]!= AddressLow )return 0;
   return 1;
}

/*
  ____________ ______ _______ __________________ ________
 |            |      |       |                  |        |
 | Address (2)| Cmd  | Psize | PayLoad( Psize ) | ChkSum |
 |____________|______|_______|__________________|________|
*/

// Sends a message 
void send_msg(unsigned int8 destH , unsigned int8 destL ,unsigned int8 cmd, unsigned int8 * payload , unsigned int8 size)
{
   unsigned int8 packet_buffer[11];
   unsigned int8 t;
   packet_buffer[0]= destH;
   packet_buffer[1]= destL;
   packet_buffer[2]= cmd;
   packet_buffer[3]= size;
   for(t=0;t<size;t++)
      {
      packet_buffer[4+t]=payload[t];
      }

   packet_buffer[4+size]=getChkSum( packet_buffer, 4+size);//reserved for checksum

   SendBytes(packet_buffer,size+5);
   
}

int1 send_msg_wack(unsigned int8 destH , unsigned int8 destL ,unsigned int8 cmd, unsigned int8 * payload , unsigned int8 size)
{
   int32 timeout;
   int8 temp;

   Restart_Receiver_FSM();
   clear_Ack_count();

   send_msg(destH ,destL ,cmd,payload ,size);
   timeout=GET_SYSTEM_TIME()+30;
      while(GET_SYSTEM_TIME()< timeout)
      {
      if(kbhit())
         {
         temp = getc();
         Receiver_FSM(temp);
            if( get_Ack_count() )
            {
         return 1;
            }
         }
      }
  return 0;
}


