//
// simple_serial_mac.h
// rev1. Mon, 13 Sep 2010 00:01:39 -0300 - first release
//
// DiegoSpinola @ Hackeneering.com
//
// Implements a simple MAC

// Returns the ChkSum for the buffer 
unsigned int8 getChkSum( unsigned int8 * buffer , int8 size)
{
   unsigned int8 t;
   unsigned int8 chk=0x00;
   for( t=0;t<size;t++)
   {
    chk^=(buffer[t]);
   }
   return ~chk;
}

void SendBytes(unsigned int8 * buffer,unsigned int8 size)
{
   int8 t=0;
   for(t=0;t<size;t++)
   {
      putc(buffer[t]);
   }
}

//Receiver FSM
/*
  ____________ ______ _______ __________________ ________
 |            |      |       |                  |        |
 | Address (2)| Cmd  | Psize | PayLoad( Psize ) | ChkSum |
 |____________|______|_______|__________________|________|
*/
#define MAX_PACKET_SIZE 16
#define MAX_PAYLOAD_SIZE 11

#define STATE_IDLE     0
#define STATE_ADD2     1
#define STATE_CMD      2
#define STATE_PSIZE    3
#define STATE_PAYLOAD  4
#define STATE_CHKSUM   5
#define FSM_TIMEOUT 200

static unsigned int8 state=STATE_IDLE;

void Restart_Receiver_FSM()
{
    state=STATE_IDLE;
}

void Receiver_FSM(unsigned int8 in)
{
   static unsigned int32 packet_timeout_due=0;

   static unsigned int8 pcursor=0;
   static unsigned int8 packet_buffer[MAX_PACKET_SIZE];

   if( state !=STATE_IDLE && packet_timeout_due < GET_SYSTEM_TIME())
   {
      //timeout
      //putc('T');
      state=STATE_IDLE;
   }
   switch(state)
   {
      case STATE_IDLE:
         packet_buffer[ADD_POS]= in;
         state =STATE_ADD2;
      break;
      case STATE_ADD2:
         packet_buffer[ADD_POS+1]= in;
         state =STATE_CMD;
      break;
      case STATE_CMD:
         packet_buffer[CMD_POS]= in;
         state =STATE_PSIZE;
      break;
      case STATE_PSIZE:
         packet_buffer[PSI_POS]= in;
         if(MAX_PAYLOAD_SIZE < in)
         {
         state = STATE_IDLE; //go back to the start, this packet is too big!
         }else{
       pcursor = 0;
       if(in==0){
       state = STATE_CHKSUM;
      
       }else{

       state = STATE_PAYLOAD;
      }
         }
      break;
      case STATE_PAYLOAD:
         packet_buffer[PAY_POS+pcursor]=in;
         pcursor++;
         if(pcursor >= packet_buffer[PSI_POS])
         {//moveon
            state = STATE_CHKSUM;
         }
      break;

      case STATE_CHKSUM:

    if( in == getChkSum( packet_buffer , PAY_POS+pcursor) ) 
    {
           packet_handler(packet_buffer); //handle packet here
    }else{
     
       output_toggle(PIN_A4);//TODO:Take this out
    }
         state = STATE_IDLE;

      break;
   }
   
   packet_timeout_due = GET_SYSTEM_TIME() + FSM_TIMEOUT; //calculate next timeout tick
   return;
}
